#include "mbed.h"
#include "TS_DISCO_F746NG.h"
#include "LCD_DISCO_F746NG.h"
#include <vector>

#include "Missile.hpp"

#include "SPI.h"
#include "LSM6DS3.h"
#include "math.h"
#include <string>

#define MEGA 1000000

// PinOut STM32F7 : https://www.google.com/search?q=stm32f7+disco+pinout&rlz=1C1CHBF_frFR824FR824&tbm=isch&source=iu&ictx=1&fir=E2dhC7XilOxSYM%253A%252CIbeBR6YmiDSP4M%252C_&vet=1&usg=AI4_-kQwPY3iMrtwipoxPA_FWFULpN7kmA&sa=X&ved=2ahUKEwiForOIicjiAhWQx4UKHYgcB1cQ9QEwAHoECAAQBA#imgrc=E2dhC7XilOxSYM:
// datasheet : https://www.st.com/resource/en/datasheet/lsm6ds3.pdf
// ex objet (Crash) : https://os.mbed.com/users/5hel2l2y/code/LSM6DS3_Demo/file/c450734baa8b/main.cpp/

LCD_DISCO_F746NG lcd;
TS_DISCO_F746NG ts;

// *decal et init*
SPI spi(PB_15, PB_14, PI_1);    // mosi (SDA), miso (SDO), sclk (SCK)
DigitalOut cs(PI_0);            // SS
DigitalIn interrupt1(PA_8);     // pin interruption int1
DigitalIn interrupt2(PA_15);    // pin interruption int2
 
//LSM6DS3 imu(PB_15, PI_1);       // SDA(MOSI), CLK

DigitalOut led1(LED1);

// protos :
void gestionAffichageTir (int x, int y, uint32_t color);
unsigned char SPI_com (unsigned char sens, unsigned char registre, unsigned char data, string  nom);
void init_reg(void);
void test_com_whoami_lms6ds3(void);
void init_Lsm6ds3(void);

// globals :
uint8_t text[30];
unsigned char whoami = 0;

// var registres :
unsigned char C_TRL1_XL = 0;        // ~= noms des registres ...
unsigned char C_TRL2_G = 0;         // ...
unsigned char C_TRL3_C = 0;
unsigned char C_TRL5_C = 0;
unsigned char C_TRL9_XL = 0;
unsigned char C_TRL10_C = 0;
unsigned char _ORIENT_CFG_G = 0;
unsigned char FUNC_CFG_ACCESS = 0;

bool ennemi1 = true;
bool ennemi2 = true;
bool ennemi3 = true;

using namespace std;

// main
int main()
{
    // decla:
    TS_StateTypeDef TS_State;   
    
    // vector <Missile> vect;         // creer les emplacements de tir       // https://www.codeguru.com/cpp/cpp/cpp_mfc/stl/article.php/c4027/C-Tutorial-A-Beginners-Guide-to-stdvector-Part-1.htm        
    Missile tir;
    
    // Affichage infos init
    lcd.SetBackColor(LCD_COLOR_BLACK);
    lcd.SetTextColor(LCD_COLOR_WHITE);
    lcd.SetFont(&Font12);
    sprintf((char*)text, "Init LSM6DS3");
    lcd.DisplayStringAt(0, LINE(0), (uint8_t *)&text, LEFT_MODE);
        
    //init_Lsm6ds3();
   
    lcd.Clear(LCD_COLOR_BLACK);
    
    /// jeu/partie en cours
    while(1)
    {
        // aff ennemi
        lcd.SetTextColor(LCD_COLOR_WHITE);
        lcd.DrawCircle(440, 100, 10);
        lcd.DrawCircle(330, 180, 10);
        lcd.DrawCircle(370, 210, 10);
    
        // aff vaiseau
        lcd.SetTextColor(LCD_COLOR_BLUE);
        lcd.DrawLine(0, 147, 15, 160);
        lcd.DrawLine(0, 172, 15, 160);
        lcd.DrawLine(0, 147, 0, 172);
        
        // detect d'un appuis 
        ts.GetState(&TS_State);
        if(TS_State.touchDetected)              // => new Tir
        {
            lcd.Clear(LCD_COLOR_BLACK);            
            
            /*  test avec Vector    => ECHEC
            for (int i = 0; i < vect.size(); i++)
            {
                if(vect.size() == 0 || vect[i].flagTir == false)    // si pas de tir en cours
                {
                    vect.push_back(TS_State.touchY[0]);             // creation nv Missile a l'emplacement du Click
                    break;
                }
            }
            */
            
            if(!tir.flagTir)       // new missile
            {                
                tir.init(TS_State.touchY[0]);
            }
        }
        
        // missile actif
        if(tir.flagTir)
        {            
            if(tir.updateTir())                
                gestionAffichageTir(tir.posX, tir.posY, LCD_COLOR_RED);         // deplacement du missile
            else
                gestionAffichageTir(tir.posX, tir.posY, LCD_COLOR_BLACK);       // disparition du missile            
                
            /*
            if(((tir.posX) + 30) >= 440 && !ennemi1) 
            {
                ennemi1 = false;
                gestionAffichageTir(tir.posX, tir.posY, LCD_COLOR_BLACK);
                lcd.SetTextColor(LCD_COLOR_BLACK);
                lcd.DrawCircle(440, 100, 10);
            }
            else
            {
                 lcd.SetTextColor(LCD_COLOR_WHITE);
                 lcd.DrawCircle(440, 100, 10);
            }
               
            if(((tir.posX + 30) >= 330) && !ennemi2) 
            {
                ennemi2 = false;
                gestionAffichageTir(tir.posX, tir.posY, LCD_COLOR_BLACK);
                lcd.SetTextColor(LCD_COLOR_BLACK);
                lcd.DrawCircle(330, 180, 10);
            }
            else 
            {
                 lcd.SetTextColor(LCD_COLOR_WHITE);
                 lcd.DrawCircle(330, 180, 10);
            }
            
            if(((tir.posX + 30) >= 370) && !ennemi3) 
            {
                ennemi3 = false;
                gestionAffichageTir(tir.posX, tir.posY, LCD_COLOR_BLACK);
                lcd.SetTextColor(LCD_COLOR_BLACK);
                lcd.DrawCircle(370, 210, 10);
            }
            else 
            {
                 lcd.SetTextColor(LCD_COLOR_WHITE);
                 lcd.DrawCircle(370, 210, 10);
            }
            */
        }
        
        
        /*  test avec Vector    => ECHEC
        for (int i = 0; i < nbrTirMax; i++)
        {
            if(vect[i].flagTir)
            {
                if(vect[i].updateTir());        // deplacement des Tirs
                {
                    gestionAffichageTir(vect[i].posX, vect[i].posY);                
                    
                    // Affichage infos
                    lcd.SetBackColor(LCD_COLOR_BLACK);
                    lcd.SetTextColor(LCD_COLOR_WHITE);
                    lcd.SetFont(&Font12);
                    sprintf((char*)text, "Touches: I: %d", i);
                    lcd.DisplayStringAt(0, LINE(0), (uint8_t *)&text, LEFT_MODE);
                    wait_ms(200);
                }
            }
            else vect.erase(vect.begin()+i);
        }
        */
        
        led1 = !led1;
    }
}


void gestionAffichageTir (int x, int y, uint32_t color)
{
    lcd.Clear(LCD_COLOR_BLACK);
    lcd.SetTextColor(color);
    lcd.DrawRect(x, y, 30, 3);     // X, Y, W, H -> effectue le Tir

    // Affichage infos
    lcd.SetBackColor(LCD_COLOR_BLACK);
    lcd.SetTextColor(LCD_COLOR_WHITE);
    lcd.SetFont(&Font12);
    sprintf((char*)text, "Touches: X: %d Y: %d", x, y);
    lcd.DisplayStringAt(0, LINE(0), (uint8_t *)&text, LEFT_MODE);
    
    wait_ms(20);
}



// SPI ***************************************
unsigned char SPI_com (unsigned char sens, unsigned char registre, unsigned char data, string nom)
{
    // -decla et init-
    unsigned char retour = 0, message = 0;
        
    // -actions-
    while (cs==0);      // verif ligne non occupée
    wait_us(2);
    
    if ((sens > 1) || (sens < 0)) return 0xFE;                  // verif bit R/W uniquement
    if ((registre > 0x6B) || (registre < 0)) return 0xFD;       // verif registre existant
    
    message = (((unsigned char)sens)<<7) | ((unsigned char)registre);   // creation du message envoyé
//    pc.printf("Envoi:: sens: %u ,registre: %u , data: %u\n", sens, registre, data);       // aff données msg total
    
    cs = 0;   // slave comm
    
    spi.write(message);         // comm SPI R/W + registre
    wait_us(2);
    retour = spi.write(data);   // comm SPI données
    wait_us(2);
//    pc.printf(" .. %s reponse SPI = %u (0x%x)\n", nom, retour, retour);       // aff val de retour de la comm SPI

    cs = 1;   // Deselect the device
    wait_us(2);
    
    // -val de retour-
    return retour;
}


// reglage des registres  
void init_reg(void)
{    
//   wait_us(3);      
//   FUNC_CFG_ACCESS = SPI_com(0,0x01,0x80,"FUNC_CFG_ACCESS");   // enable acces to configuration registers functions 02h to 32h/ 0(7b)
    wait_us(3);
    C_TRL1_XL = SPI_com(0,0x10,0x84,"CTRL1_XL");     // accelero select  |  [208HZ] 1.66KHz(4b)/ 16g[8g](2b)/ [100Hz] 400Hz(2b)  => data = 5E \ 8C
    wait_us(3);
    C_TRL2_G = SPI_com(0,0x11,0x8C,"CTRL2_G");       // gyro select  |  [208HZ] 1.66KHz(4b)/ 2000dps(2b)/ diasable at 125dps(2b) => data = 5C
//    wait_us(3);
//    C_TRL3_C = SPI_com(0,0x12,0x06,"CTRL3_C");       // control register |  reboot memory/ BDU/ interrupt active high/ PP-OP/ 3-wire SPI/ IF_INC enable/ BIG endian/ SfWr_not_RESET
    wait_us(3);
    _ORIENT_CFG_G = SPI_com(0,0x0B,0x18,"ORIENT_CFG_G");     // 0(2b)/ signe par default (X-,Y+,Z+)(3b)/ direction (3b) 
    wait_us(3);
    C_TRL5_C = SPI_com(0,0x14,0x00,"CTRL5_C");       // | nop [self-test gyro + accelero]/ 0/ G self_test/ A slef_test => data = 6F | 60
    wait_us(3);
    C_TRL9_XL = SPI_com(0,0x18,0x38,"CTRL9_XL");     // accelero out enable   |   0/ 0/ accelero axe Z-Y-X(3b)/ 0(3b)
    wait_us(3);
    C_TRL10_C = SPI_com(0,0x19,0x38,"CTRL10_C");     // gyro out enable    |   0/ 0/ gyro Z-Y-X(3b)/ 0(3b)
    wait_us(3);
    // CTRL6_C
    // CTRL7_G  
}


void test_com_whoami_lms6ds3(void)
{
    lcd.SetBackColor(LCD_COLOR_BLACK);
    lcd.SetTextColor(LCD_COLOR_WHITE);
    lcd.SetFont(&Font12);
    
    // test si la carte est reconnue *OK*
    whoami = SPI_com(1,0x0F,0,"whoami");            // param => W=0|R=1(1b) / registre(7b) / data(8b)
    if (whoami != 0x69)        
        sprintf((char*)text, " SPI FAIL, je suis: %u et pas \"0x69 => 105\"", whoami);          // aff resultat test ECHEC          
    else sprintf((char*)text, " SPI SUCCES, je suis bien %u : 0x69",whoami);                    // aff resultat test BON                      
    
    lcd.DisplayStringAt(0, LINE(2), (uint8_t *)&text, LEFT_MODE);
    wait(1);
}


void init_Lsm6ds3(void)
{
    /* => 0 reponses
    uint16_t status = imu.begin(imu.G_SCALE_2000DPS, imu.A_SCALE_8G, imu.G_ODR_1660, imu.A_ODR_6660);
    // Affichage infos
    lcd.SetBackColor(LCD_COLOR_BLACK);
    lcd.SetTextColor(LCD_COLOR_WHITE);
    lcd.SetFont(&Font12);
    sprintf((char*)text, "WhoAmI (0x69 ?) : 0x%X", status);
    lcd.DisplayStringAt(0, LINE(0), (uint8_t *)&text, LEFT_MODE);
    */                     
    
    // init comm SPI    
    cs = 1;                            // pas de slave
    spi.format(8,0);                   // spi.format = 8 bits, mode 0)
    spi.frequency(1 * MEGA);           // 1MHz max                     | change val de rep     => 5MHz ok

    test_com_whoami_lms6ds3();  // test whoami -> 0x69

    init_reg();         // config registre
}
